% 25/06/2009
% Verification of the performance of the neural network
% considering training, validation and test set
% Use of the weights obtained after training the neural network
% Presentation of graphical results
%
clear all;format compact;format long;
disp('(1) Use weights minimizing the validation error');
disp('(2) Use weights minimizing the training error');
opt = input('Which set of weights you would like to use? ');
if opt == 1,
    load w1v;load w2v;
elseif opt == 2,
    load w1;load w2;
else
    error('Not a valid option!');
end
load evol;
load train; % training dataset
load valid; % validation dataset
load test; % test dataset
N = length(X(:,1));
X = [X ones(N,1)];
Nv = length(Xv(:,1));
Xv = [Xv ones(Nv,1)];
Nt = length(Xt(:,1));
Xt = [Xt ones(Nt,1)];
n(1,1) = length(w1(:,1));
n(2,1) = length(w2(:,1));
r = n(2);
m = length(w1(1,:))-1;
np1 = n(1)*(m+1);np2 = n(2)*(n(1)+1);
npesos = np1+np2;
disp(sprintf('No. of training patterns = %d',N));
disp(sprintf('No. of validation patterns = %d',Nv));
disp(sprintf('No. of test patterns = %d',Nt));
disp(sprintf('No. of hidden layer neurons = %d',n(1)));
disp(sprintf('No. of weights = %d',npesos));
disp(sprintf('No. of iterations for the training = %d',niter));
disp(sprintf('No. of iterations that minimizes the validation error = %d',niter_v));
Srn = [tanh(X*w1') ones(N,1)]*w2';
verro = reshape(S-Srn,N*r,1);
eqmtr = sqrt((1/(N*r))*(verro'*verro));
disp(sprintf('Final mean squared error (training) = %.12g',eqmtr));
[win_value_NNtr,class_NNtr] = max(Srn');
[win_valuetr,classtr] = max(S');
count = 0;
ind_errortr = [];
for i=1:N,
    if abs(classtr(i)-class_NNtr(i))>0.1,
        count = count+1;
        ind_errortr = [ind_errortr,i];
    end
end
ratetr = count/N;
disp(sprintf('Final rate of classification error (training) = %.12g',ratetr));
disp('Patterns not correctly classified (training)');
if length(ind_errortr) > 0,
    disp(ind_errortr);
else
    disp('[]');
end
erro = S-Srn;
if r > 1,
    for k=1:r,
        veqm(k,1) = sqrt((1/N)*(erro(:,k)'*erro(:,k)));
        disp(sprintf('Final mean squared error (output %d) (training) = %.12g',k,veqm(k,1)));
    end
end
for k=1:r,
	figure(k);subplot(111);plot(S(:,k),'r');hold on;plot(S(:,k),'or');
	plot(Srn(:,k),'g');plot(Srn(:,k),'og');hold off;
    title(sprintf('Output %d: Desired (r)  NN (g) - Training',k));
end
for k=(r+1):(r+r),
	figure(k);subplot(111);plot(S(:,k-r)-Srn(:,k-r),'r');
	hold on;plot(S(:,k-r)-Srn(:,k-r),'or');hold off;
	title(sprintf('Output %d: Training Error',k-r));
end
for k=1:r,
	Smedio(1,k) = (1/N)*S(:,k)'*ones(N,1);
end
den = 0;num = 0;
for l=1:N,
	for k=1:r,
		num = num+(S(l,k)-Srn(l,k))^2;
		den = den+(S(l,k)-Smedio(1,k))^2;
	end
end
fvu = num/den;
disp(sprintf('FUV (training) = %.12g',fvu));
figure(r+r+1);subplot(111);semilogy(eq);title('EQ - Training');
figure(r+r+2);subplot(211);plot(stw1);title('Change in weights along training');subplot(212);plot(stw2);
figure(r+r+3);plot(rms_w);title('Root mean squared value of the weights along training');
mean_S = (S'*ones(N,1))/N;
mean_Srn = (S'*ones(N,1))/N;
M_mean_S = [];
M_mean_Srn = [];
for i=1:r,
    M_mean_S = [M_mean_S mean_S(i).*ones(N,1)];
    M_mean_Srn = [M_mean_Srn mean_Srn(i).*ones(N,1)];
end
dp_S = sqrt((((S-M_mean_S).^2)'*ones(N,1))./N);
dp_Srn = sqrt((((Srn-M_mean_Srn).^2)'*ones(N,1))./N);
cov = (((S-M_mean_S).*(Srn-M_mean_Srn))'*ones(N,1))/N;
cor = cov./(dp_S.*dp_Srn);
R2 = cor.^2;
for i=1:r,
    disp(sprintf('Output %d: Correlation (training) = %.12g',i,cor(i)));
    disp(sprintf('Output %d: Factor R2 (training) = %.12g',i,R2(i)));
end

n_fig = 3+2*r;
Srnv = [tanh(Xv*w1') ones(Nv,1)]*w2';
verrov = reshape(Sv-Srnv,Nv*r,1);
eqmv = sqrt((1/(Nv*r))*(verrov'*verrov));
disp(sprintf('Final mean squared error (validation) = %.12g',eqmv));
[win_value_NNv,class_NNv] = max(Srnv');
[win_valuev,classv] = max(Sv');
count = 0;
ind_errorv = [];
for i=1:Nv,
    if abs(classv(i)-class_NNv(i))>0.1,
        count = count+1;
        ind_errorv = [ind_errorv,i];
    end
end
ratev = count/Nv;
disp(sprintf('Final rate of classification error (validation) = %.12g',ratev));
disp('Patterns not correctly classified (validation)');
if length(ind_errorv) > 0,
    disp(ind_errorv);
else
    disp('[]');
end
errov = Sv-Srnv;
if r > 1,
	for k=1:r,
        veqmv(k,1) = sqrt((1/Nv)*(errov(:,k)'*errov(:,k)));
        disp(sprintf('Final mean squared error (output %d) (validation) = %.12g',k,veqmv(k,1)));
    end
end
for k=1:r,
	figure(n_fig+k);subplot(111);plot(Sv(:,k),'r');hold on;plot(Sv(:,k),'or');
	plot(Srnv(:,k),'g');plot(Srnv(:,k),'og');hold off;
	title(sprintf('Output %d: Desired (r)  NN (g) - Validation',k));
end
for k=(r+1):(r+r),
	figure(n_fig+k);subplot(111);plot(Sv(:,k-r)-Srnv(:,k-r),'r');
	hold on;plot(Sv(:,k-r)-Srnv(:,k-r),'or');hold off;
	title(sprintf('Output %d: Validation error',k-r));
end
for k=1:r,
	Smediov(1,k) = (1/Nv)*Sv(:,k)'*ones(Nv,1);
end
den = 0;num = 0;
for l=1:Nv,
	for k=1:r,
		num = num+(Sv(l,k)-Srnv(l,k))^2;
		den = den+(Sv(l,k)-Smediov(1,k))^2;
	end
end
fvuv = num/den;
disp(sprintf('FUV (validation) = %.12g',fvuv));
figure(n_fig+r+r+1);subplot(111);semilogy(eqv);title('EQ - Validation');
mean_Sv = (Sv'*ones(Nv,1))/Nv;
mean_Srnv = (Sv'*ones(Nv,1))/Nv;
M_mean_Sv = [];
M_mean_Srnv = [];
for i=1:r,
    M_mean_Sv = [M_mean_Sv mean_Sv(i).*ones(Nv,1)];
    M_mean_Srnv = [M_mean_Srnv mean_Srnv(i).*ones(Nv,1)];
end
dp_Sv = sqrt((((Sv-M_mean_Sv).^2)'*ones(Nv,1))./Nv);
dp_Srnv = sqrt((((Srnv-M_mean_Srnv).^2)'*ones(Nv,1))./Nv);
covv = (((Sv-M_mean_Sv).*(Srnv-M_mean_Srnv))'*ones(Nv,1))/Nv;
corv = covv./(dp_Sv.*dp_Srnv);
R2v = corv.^2;
for i=1:r,
    disp(sprintf('Output %d: Correlation (validation) = %.12g',i,corv(i)));
    disp(sprintf('Output %d: Factor R2 (validation) = %.12g',i,R2v(i)));
end

n_fig = 6+4*r;
Srnt = [tanh(Xt*w1') ones(Nt,1)]*w2';
verrot = reshape(St-Srnt,Nt*r,1);
eqt = 0.5*(verrot'*verrot);
eqmt = sqrt((1/(Nt*r))*(verrot'*verrot));
disp(sprintf('Final mean squared error (test) = %.12g',eqmt));
[win_value_NNt,class_NNt] = max(Srnt');
[win_valuet,classt] = max(St');
count = 0;
ind_errort = [];
for i=1:Nt,
    if abs(classt(i)-class_NNt(i))>0.1,
        count = count+1;
        ind_errort = [ind_errort,i];
    end
end
ratet = count/Nt;
disp(sprintf('Final rate of classification error (test) = %.12g',ratet));
disp('Patterns not correctly classified (test)');
if length(ind_errort) > 0,
    disp(ind_errort);
else
    disp('[]');
end
errot = St-Srnt;
if r > 1,
	for k=1:r,
        veqmt(k,1) = sqrt((1/Nt)*(errot(:,k)'*errot(:,k)));
        disp(sprintf('Final mean squared error (output %d) (test) = %.12g',k,veqmt(k,1)));
    end
end
for k=1:r,
	figure(n_fig+k);subplot(111);plot(St(:,k),'r');hold on;plot(St(:,k),'or');
	plot(Srnt(:,k),'g');plot(Srnt(:,k),'og');hold off;
    title(sprintf('Output %d: Desired (r)  NN (g) - Test',k));
end
for k=(r+1):(r+r),
	figure(n_fig+k);subplot(111);plot(St(:,k-r)-Srnt(:,k-r),'r');
	hold on;plot(St(:,k-r)-Srnt(:,k-r),'or');hold off;
	title(sprintf('Output %d: Test error',k-r));
end
for k=1:r,
	Smediot(1,k) = (1/Nt)*St(:,k)'*ones(Nt,1);
end
den = 0;num = 0;
for l=1:Nt,
	for k=1:r,
		num = num+(St(l,k)-Srnt(l,k))^2;
		den = den+(St(l,k)-Smediot(1,k))^2;
	end
end
fvut = num/den;
disp(sprintf('FUV (test) = %.12g',fvut));
mean_St = (St'*ones(Nt,1))/Nt;
mean_Srnt = (St'*ones(Nt,1))/Nt;
M_mean_St = [];
M_mean_Srnt = [];
for i=1:r,
    M_mean_St = [M_mean_St mean_St(i).*ones(Nt,1)];
    M_mean_Srnt = [M_mean_Srnt mean_Srnt(i).*ones(Nt,1)];
end
dp_St = sqrt((((St-M_mean_St).^2)'*ones(Nt,1))./Nt);
dp_Srnt = sqrt((((Srnt-M_mean_Srnt).^2)'*ones(Nt,1))./Nt);
covt = (((St-M_mean_St).*(Srnt-M_mean_Srnt))'*ones(Nt,1))/Nt;
cort = covt./(dp_St.*dp_Srnt);
R2t = cort.^2;
for i=1:r,
    disp(sprintf('Output %d: Correlation (test) = %.12g',i,cort(i)));
    disp(sprintf('Output %d: Factor R2 (test) = %.12g',i,R2t(i)));
end
figure(n_fig+2*r+1);subplot(111);plot(eqv,'k');title(sprintf('Rate of classification error (validation) - Minimum at it = %d',niter_v));
